local cx,cy = math.random(-10,10)/10000,math.random(-10,10)/10000

local tile_size = 42
local line_it = 6
local line_len = 12
local ts2 = tile_size/2

local part = {}

local ScrW, ScrH = love.graphics.getWidth, love.graphics.getHeight

for I = 1, 24 do
	part[I] = {
		x = math.random(ScrW()),
		y = math.random(ScrH()),

		r = 0,
		--m = math.random(2,8),--+math.random(9000)/1000,

		ax = 0,
		ay = 0,
		ax2 = 0,
		ay2 = 0,

		vx = 0,
		vy = 0,

		--sx = 20,
		--sy = 20,
		lpos = {},
		li = 1,

		cr = math.random(255),
		cg = math.random(255),
		cb = math.random(255),
	}

	part[I].m = (math.random(1400,math.random(1500,3000))/1000)^2


	part[I].sx = 5+part[I].m*3
	part[I].sy = part[I].sx
end

local addVel

local rx,ry = math.random(-1000000,1000000)/1000,math.random(-1000000,1000000)/1000
local noise = function(x,y) 
	return love.math.noise(x/800+rx,y/800+ry)
end

do
	local sin,cos,pi2 = math.sin, math.cos, math.pi*2

	local f = 70

	function addVel(p,dt)
		local n = noise( p.x , p.y )
		--p.r = n*pi2

		local mx,my = cos(n*pi2)+p.ax2/2,(sin(n*pi2)+p.ay2)/2

		p.ax = (mx/p.m)*f
		p.ay = (my/p.m)*f

		p.vx = p.vx + p.ax
		p.vy = p.vy + p.ay

		p.vx = p.vx - (p.vx/p.m^2)/4
		p.vy = p.vy - (p.vy/p.m^2)/4

		p.r = math.atan(p.vy/p.vx)

		--local v = ((p.x^2)+(p.y^2))^0.5

		--p.x = p.x + mx*dt*v
		--p.y = p.y + my*dt*v
	end
end

local drag_x,drag_y = 0,0
local drag = false

function love.update(dt)
	if drag and not love.mouse.isDown(1) then
		drag = false
	elseif not drag and love.mouse.isDown(1)then
		drag = true
		drag_x = love.mouse.getX()
		drag_y = love.mouse.getY()
	end

	for k,v in ipairs(part) do
		addVel(v,dt)

		v.lpos[v.li] = {v.x,v.y,v.r,v.cr,v.cg,v.cb}
		v.li = v.li + 1
		if v.li > 20 then v.li = 1 end

		v.x = v.x + v.vx*dt
		v.y = v.y + v.vy*dt

		if v.x > ScrW() then v.x = 0 end
		if v.y > ScrH() then v.y = 0 end

		if v.x < 0 then v.x = ScrW() end
		if v.y < 0 then v.y = ScrH() end
	end
end

local rect = love.graphics.rectangle
local line = love.graphics.line
local color = love.graphics.setColor
local fill = "fill"
local _line = "line"
local col = {255,160,0,180}

local push, pop = love.graphics.push, love.graphics.pop
local translate, rotate = love.graphics.translate, love.graphics.rotate
local min,max = math.min, math.max
local sin,cos,pi2 = math.sin, math.cos, math.pi*2

local function slope(n)
		return max(0,min(1,1-4*(n-0.5)^(2)))
end
local function drawRect(p,x,y,sx,sy,sx2,sy2,dt)
	local N = noise( x , y )

	local gx,gy = p.x,p.y
	love.graphics.setBlendMode("add")
	color(p.cr,p.cg,p.cb,20)
	local it = math.ceil(p.m+1)
	for I= 1,it do
		local lx,ly = gx,gy
		local N2 = noise( lx , ly )
		local S2 = slope(N2)
		--color(sin(N2*pi2) * 128 + 127,sin(N2*pi2+pi2/6) * 128 + 127,sin(N2*pi2+pi2/3) * 128 + 127,180)
		gx, gy = gx+cos(N2*pi2)*5*(S2+1),gy+sin(N2*pi2)*5*(S2+1)
		
		if I == it then
			local _cr,_cg,_cb = sin(N*pi2) * 128 + 127,sin(N*pi2+pi2/6) * 128 + 127,sin(N*pi2+pi2/3) * 128 + 127

			p.cr = p.cr + ((_cr - p.cr))*dt*4
			p.cg = p.cg + ((_cg - p.cg))*dt*4
			p.cb = p.cb + ((_cb - p.cb))*dt*4

			p.ax2 = cos(N2*pi2)*S2
			p.ay2 = sin(N2*pi2)*S2

			push()
				translate(lx, ly)
				rotate(N2*pi2)
				rect(fill,-sx2,-sy2,sx,sy)
			pop()

		end

		push()
			translate(lx, ly)
			rotate(N2*pi2)
			rect(_line,-sx2,-sy2,sx,sy)
		pop()

	end
	love.graphics.setBlendMode("alpha")

	local _cr,_cg,_cb = sin(N*pi2) * 128 + 127,sin(N*pi2+pi2/6) * 128 + 127,sin(N*pi2+pi2/3) * 128 + 127

	p.cr = p.cr + ((_cr - p.cr))*dt*4
	p.cg = p.cg + ((_cg - p.cg))*dt*4
	p.cb = p.cb + ((_cb - p.cb))*dt*4

	color(p.cr,p.cg,p.cb,255)
	push()
		
		translate(p.x,p.y)
		rotate(p.r)
		love.graphics.setBlendMode("subtract")
		rect(fill,-sx2-1,-sy2-1,sx+2,sy+2)
		love.graphics.setBlendMode("alpha")
	pop()

	for I,pos in ipairs(p.lpos) do
		--color(255,255,255,220*(((I-1+v.li)%9+1)/10))
		push()
			translate(pos[1],pos[2])
			rotate(pos[3])
			color(pos[4],pos[5],pos[6],20)
			rect(fill,-sx2,-sy2,sx,sy)
		pop()
	end

	color(p.cr,p.cg,p.cb,255)
	push()
		
		translate(p.x,p.y)
		rotate(p.r)
		
		rect(fill,-sx2,-sy2,sx,sy)
		--translate(p.x,p.y)
		--rotate(-p.r)
	pop()
end

local function drawParticle(p,dt)
	local sx2, sy2 = p.sx/2, p.sy/2

	color(col)
	drawRect(p, p.x-sx2, p.y-sy2, p.sx, p.sy, sx2, sy2,dt)

	if p.x + sx2 >= ScrW() then
		drawRect(p, p.x-sx2-ScrW(), p.y-sy2, p.sx, p.sy, sx2, sy2,dt)
	elseif p.y + sy2 >= ScrH() then
		drawRect(p, p.x-sx2, p.y-sy2-ScrH(), p.sx, p.sy, sx2, sy2,dt)
	elseif p.x - sx2 <= 0 then
		drawRect(p, p.x-sx2+ScrW(), p.y-sy2, p.sx, p.sy, sx2, sy2,dt)
	elseif p.y - sy2 <= 0 then
		drawRect(p, p.x-sx2, p.y-sy2+ScrH(), p.sx, p.sy, sx2, sy2,dt)
	end
end

local time = love.timer.getTime
local t = time()

local canvas
local setcanvas,draw = love.graphics.setCanvas, love.graphics.draw

function movecanvas(x,y)
	rx = rx+ x
	ry = ry+ y
	canvas = nil
end

local rnd = math.random

function love.draw()

	dt = time() - t
	t = time()

	if not canvas then
		canvas = love.graphics.newCanvas( )
		setcanvas(canvas)
		love.graphics.clear( )
		for X = 0,math.ceil(ScrW()/tile_size)-1 do
			for Y = 0,math.ceil(ScrH()/tile_size)-1 do
				local px,py = ((X*tile_size)+ts2), ((Y*tile_size)+ts2)
				local N = noise( px , py )
				local S = slope( N )


				color(255,255,255,100)
				--color(sin(N*pi2) * 128 + 127,sin(N*pi2+pi2/6) * 128 + 127,sin(N*pi2+pi2/3) * 128 + 127,100)
				rect(fill,px-ts2,py-ts2,tile_size,tile_size)
				--color(sin(N*pi2) * 128 + 127,sin(N*pi2+pi2/6) * 128 + 127,sin(N*pi2+pi2/3) * 128 + 127,255)
				color(sin(N*pi2) * 128 + 127,sin(N*pi2+pi2/6) * 128 + 127,sin(N*pi2+pi2/3) * 128 + 127,120)
				rect(fill,px-2,py-2,4,4)

				local gx, gy = px+cos(N*pi2)*(line_len+rnd(-1,1))*(S+1),py+sin(N*pi2)*(line_len+rnd(-1,1))*(S+1)

				--color(sin(N*pi2) * 128 + 127,sin(N*pi2+pi2/6) * 128 + 127,sin(N*pi2+pi2/3) * 128 + 127,200+55*S)
				--line(gx, gy,px+cos(N*pi2)*5*S,py+sin(N*pi2)*5*S)
				
				love.graphics.setBlendMode("add")
				line(gx, gy,px,py)
				for I= 1,rnd(line_it*0.67,line_it) do
					local lx,ly = gx,gy
					local N2 = noise( lx , ly )
					local S2 = slope(N2)
					color(sin(N2*pi2) * 128 + 127,sin(N2*pi2+pi2/6) * 128 + 127,sin(N2*pi2+pi2/3) * 128 + 127,120)
					gx, gy = gx+cos(N2*pi2)*(line_len+rnd(-1,1))*(S2+1),gy+sin(N2*pi2)*(line_len+rnd(-1,1))*(S2+1)
					line(lx, ly,gx,gy)
				end
				love.graphics.setBlendMode("alpha")
			end
		end
		setcanvas()
	end
	if canvas then
		color(255,255,255,150)
		draw(canvas)
		movecanvas(cx,cy)
		if drag then
			cx = -(love.mouse.getX()-drag_x)/ScrW()*2
			cy = -(love.mouse.getY()-drag_y)/ScrH()*2
			drag_x = love.mouse.getX()
			drag_y = love.mouse.getY()
		end
	end
	for k,v in ipairs(part) do
		drawParticle(v,dt)
	end
end
